package jrummikub.util;

/**
 * A pair of objects
 * 
 * @param <T1>
 *            Type of first component
 * @param <T2>
 *            Type of second component
 */
public class Pair<T1, T2> {
	private final T1 first;
	private final T2 second;

	/**
	 * Create a new pair from two values
	 * 
	 * @param first
	 *            the first pair component
	 * @param second
	 *            the second pair component
	 */
	public Pair(T1 first, T2 second) {
		this.first = first;
		this.second = second;
	}

	/**
	 * Extract the first component of a pair
	 * 
	 * @return the first pair component
	 */
	public T1 getFirst() {
		return first;
	}

	/**
	 * Extract the second component of a pair
	 * 
	 * @return the second pair component
	 */
	public T2 getSecond() {
		return second;
	}

	@Override
	public String toString() {
		return "Pair [first=" + first + ", second=" + second + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((first == null) ? 0 : first.hashCode());
		result = prime * result + ((second == null) ? 0 : second.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		@SuppressWarnings("rawtypes")
		Pair other = (Pair) obj;
		if (first == null) {
			if (other.first != null)
				return false;
		} else if (!first.equals(other.first))
			return false;
		if (second == null) {
			if (other.second != null)
				return false;
		} else if (!second.equals(other.second))
			return false;
		return true;
	}

}
